/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
  
 package de.bsvrz.ste.rdstmcvew;

import java.util.HashMap;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.NumberValue;
import de.bsvrz.dav.daf.main.Data.TextValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.InvalidationListener;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse zur Verwaltung von RDS/TMC-Meldungen einer LMS.<br>
 * Beim Instanziieren eines Klassenobjekts werden aus einem Objekt vom Typ 
 * typ.landesMeldeStelle die RDS/TMC-Meldungen der zugeordneten Menge "
 * RDSMeldungen" zur Verwaltung eingelesen. 
 * Die Mengen der aktuellen RDS/TMC-Meldungen wird bezglich neuen Elementen 
 * und Lschung vorhandener Elemente berwacht.  
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.2 $ / $Date: 2008/09/30 15:20:19 $ / ($Author: Hilpp $)
 * 
 * Geprft Hilpp 30.9.2008
 */
public class RDSVerwaltung implements MutableSetChangeListener,
  InvalidationListener, ClientReceiverInterface
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Konfigurationsobjekt der LMS
   */
  private ConfigurationObject _konfigObjekt;
  
  /**
   * Name der dynamische Menge, die fr das Konfigurationsobjekt verwendet wird 
   */
  public final static String RDSMeldungen = "RDSMeldungen";
  
  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;
    
  /**
   * Lschverfahren fr die dynamischen RDS-Meldungsobjekte
   */
  private int _loeschVerfahren;
  
  /**
   * Flag zur Unterdrckung der Korrektur, wenn ein Zeitstempel einer Meldung in der Vergangenheit liegt
   */
  private boolean  _ohneZeitstempelKorrektur;
  
  /**
   * Datenbeschreibung der ATG, die die verwendeten Parameter enthlt
   */
  private DataDescription _datenBeschreibung;
  
  /**
   * Alle aktuell verwalteten RDS/TMC-Meldungen
   */
  private Map<SystemObject, RDSVewMeldung> _aktuelleMeldungen = new HashMap<SystemObject, RDSVewMeldung>();
  
  /**
   * Instanziiert die Verwaltung fr RDS/TMC-Meldungen
   * 
   * @param connection die Datenverteiler-Verbindung
   * @param konfigObjekt das Konfigurationsobjekt, das die LMS darstellt
   * @param optionen Optionen als Bitmaske
   */
  public RDSVerwaltung(ClientDavInterface connection, ConfigurationObject konfigObjekt,
      int optionen)
  {
    _connection = connection;
    _konfigObjekt = konfigObjekt;
    _loeschVerfahren = optionen & RDSVerwaltungApp.MASKE_LOESCH_VERFAHREN; 
    _ohneZeitstempelKorrektur = (optionen & RDSVerwaltungApp.MASKE_OHNE_ZEITSTEMPEL_KORREKTUR) != 0;
    
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.rdsMeldungsVerwaltung");
    Aspect asp = _connection.getDataModel().getAspect("asp.parameterSoll");
    _datenBeschreibung = new DataDescription(atg, asp);
    _connection.subscribeReceiver(this, _konfigObjekt, _datenBeschreibung,
        ReceiveOptions.normal(), ReceiverRole.receiver());
    _debug.config(_datenBeschreibung + " zum Empfang angemeldet fr " + _konfigObjekt);
    
    MutableSet menge = _konfigObjekt.getMutableSet(RDSMeldungen);
    for (SystemObject obj: menge.getElements())
    {
      if (obj.isValid())
      {
        initialisiereMeldung(obj);
      }
    }
    menge.addChangeListener(this);
    _debug.config("An Menge \"" + RDSMeldungen + "\" angemeldet");
    
    _debug.config(_aktuelleMeldungen.size() + 
        " RDS/TMC Meldungsobjekt(e) initial verwaltet");
  }
  
  /**
   * Instanziiert die Verwaltung fr RDS/TMC-Meldungen
   * 
   * @param connection die Datenverteiler-Verbindung
   * @param konfigObjekt das Konfigurationsobjekt, das die LMS darstellt
   */
  public RDSVerwaltung(ClientDavInterface connection, ConfigurationObject konfigObjekt)
  {
    this(connection, konfigObjekt, 0);
  }
  
  /**
   * Gibt belegte Resourcen frei
   */
  public void dispose()
  {
    _konfigObjekt.getMutableSet(RDSMeldungen);
    _debug.config("Von Menge \"" + RDSMeldungen + "\" abgemeldet");
    
    for (RDSVewMeldung meldung: _aktuelleMeldungen.values())
    {
      if (meldung.getDavObjekt() instanceof DynamicObject)
      {
        ((DynamicObject)meldung.getDavObjekt()).removeListenerForInvalidation(this);
        _debug.fine("Invalidation-Listener abgemeldet fr " + 
            meldung.getDavObjekt().getPid());
      }
      meldung.dispose();
      
    }
    _aktuelleMeldungen.clear();
    
    _connection.unsubscribeReceiver(this, _konfigObjekt, _datenBeschreibung);
    _debug.config("Empfang abgemeldet von " + _datenBeschreibung + " fr " + 
        _konfigObjekt);
  }
    
  /**
   * @return die aktuelle DAV-Verbindung
   */
  public ClientDavInterface getConnection()
  {
    return _connection;
  }
  
  /**
   * @return das aktuelle Konfigurationsobjekt vom Typ typ.landesMeldeStelle
   */
  public ConfigurationObject getKonfigObjekt()
  {
    return _konfigObjekt;
  }
  
  /**
   * @return das Lschverfahren
   */
  public int getLoeschVerfahren()
  {
    return _loeschVerfahren;
  }
  /**
   * @return true, falls die Korrektur unterdrckt wird, wenn ein Zeitstempel 
   * einer Meldung in der Vergangenheit liegt, false sonst 
   */
  public boolean getOhneZeitstempelKorrektur()
  {
    return _ohneZeitstempelKorrektur;
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.config.MutableSetChangeListener#update(de.bsvrz.dav.daf.main.config.MutableSet, de.bsvrz.dav.daf.main.config.SystemObject[], de.bsvrz.dav.daf.main.config.SystemObject[])
   */
  public void update(MutableSet set, SystemObject[] addedObjects, 
      SystemObject[] removedObjects)
  {
    _debug.fine("MutableSetChangeListener.update() von \"" + set.getName() +"\"");
    
    if (addedObjects.length > 0)
    {  
      _debug.config("Dynamische Menge \"" + set.getName() + "\" um " +
          addedObjects.length + " Objekte vergrssert");
      
      for(SystemObject obj: addedObjects)
      {
        if (obj.isValid())
        {  
          initialisiereMeldung(obj);
        }  
      }
    }  
          
    if (removedObjects.length > 0)
    {  
      _debug.config("Dynamische Menge \"" + set.getName() + "\" um " +
          removedObjects.length + " Objekte verkleinert");
      
      for(SystemObject obj: removedObjects)
      {
        bereinigeMeldung(obj);
      }
    }      
  }
  
  /**
   * Bereinigt die Verwaltung fr eine neu entfernte
   * (gelschte, invalidierte) RDS/TMC-Meldung
   * 
   * @param obj Das Stellvertreter-Objekt der RDS/TMC-Meldung
   */
  private void bereinigeMeldung(SystemObject obj) 
  {
    RDSVewMeldung meldung = _aktuelleMeldungen.remove(obj);
    if (meldung != null)
    {
      meldung.dispose();
    }
  }
  
  /**
   * Bereinigt die Verwaltung fr eine neu entfernte
   * (gelschte, invalidierte) RDS/TMC-Meldung.
   * Diese Methode nimmt die Bereinigung fr die Meldung 
   * in ihrer Eigenschaft als dynmaisches Objekt vor und ruft 
   * dann {@link de.bsvrz.ste.rdstmcvew.RDSVerwaltung#bereinigeMeldung(SystemObject)}
   * auf.
   * 
   * @param obj Das dynamische Stellvertreter-Objekt der RDS/TMC-Meldung
   */
  private void bereinigeMeldung(DynamicObject obj)
  {
    obj.removeListenerForInvalidation(this);
    _debug.fine("Invalidation-Listener abgemeldet fr " + obj.getPid());
    
    /* Falls obj vor dem lschen nicht aus der dyn. Menge 
     * entfernt wurde:
     */
    bereinigeMeldung((SystemObject)obj); 
  }
  
  /**
   * Initialisiert die Verwaltung fr eine neu hinzugekommene
   * RDS/TMC-Meldung
   * 
   * @param obj Das Stellvertreter-Objekt der RDS/TMC-Meldung
   */
  private void initialisiereMeldung(SystemObject obj) 
  {
    RDSVewMeldung meldung = new RDSVewMeldung(obj, this);
    _aktuelleMeldungen.put(obj, meldung);
    if (obj instanceof DynamicObject)
    {  
      ((DynamicObject)obj).addListenerForInvalidation(this);
      _debug.fine("Invalidation-Listener angemeldet fr " + obj.getPid());
    }  
  }

  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.config.InvalidationListener#invalidObject(de.bsvrz.dav.daf.main.config.DynamicObject)
   */
  public void invalidObject(DynamicObject dynamicObject)
  {
    _debug.config("Objekt " + dynamicObject + 
        " gelscht/invalidiert => wird nicht weiter verwaltet");
    
    bereinigeMeldung(dynamicObject);
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] resultData)
  {
    for (ResultData data : resultData)
    {
      Aspect asp = data.getDataDescription().getAspect();
      AttributeGroup atg = data.getDataDescription().getAttributeGroup();
      
      if (atg.equals(_datenBeschreibung.getAttributeGroup()) &&
          asp.equals(_datenBeschreibung.getAspect())) 
      {
        if (data.hasData())
        {  
          NumberValue nv = data.getData().getUnscaledValue("RDSMeldungsRckhaltung");
          _rdsMeldungsRckhaltung = nv == null ? - 1 : nv.longValue();
          TextValue tv = data.getData().getTextValue("RDSMeldungsAktion");
          _rdsMeldungsAktion = tv == null ? null : tv.getText();
        }
        else
        {
          _rdsMeldungsRckhaltung = -1;
          _rdsMeldungsAktion = null;
        }
        _debug.config("Update Daten " + atg.getPid() + " fr " + _konfigObjekt.getName()
            + "\nRDSMeldungsRckhaltung auf " + _rdsMeldungsRckhaltung
            + "\nRDSMeldungsAktion auf " + _rdsMeldungsAktion);
      }
    }
  }
    
  /**
   * Parameter SendeVerzoegerungAufhebung
   */
  private long _rdsMeldungsRckhaltung = -1;
  
  /**
   * @return den Wert des Parameters RDSMeldungsRckhaltung
   */
  public long getRDSMeldungsRckhaltung()
  {
    return _rdsMeldungsRckhaltung;
  }
  
  /**
   * Parameter MeldungsErhaltung
   */
  private String _rdsMeldungsAktion = null;
  
  /**
   * @return den Wert des Parameters RDSMeldungsAktion
   */
  public String getRDSMeldungsAktion()
  {
    return _rdsMeldungsAktion;
  }
  
}
