/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */


package de.bsvrz.ste.rdstmcvew;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;

/**
 * Applikationsklasse der SWE RDS/TMC-Verwaltung.<br>
 * Fr die bergebene LMS wird eine Applikations-Instanz der SWE RDS/TMC 
 * erzeugt und initialisiert.<br>
 * SWE-spezifische Parameter:<br>
 * -lms: Die Landesmeldestelle, auf welche diese Instanz der SWE sich bezieht.
 * Das Konfigurationsobjekt der LMS enthlt insbesondere die zu verwendeten Paramenter.
 * 
 * @author Dambach Werke GmbH
 * @author Stefan Sans
 * @version $Revision: 1.2 $ / $Date: 2008/09/30 15:20:19 $ / ($Author: Hilpp $)
 * 
 * Geprft Hilpp 30.9.2008
 */
public class RDSVerwaltungApp implements StandardApplication
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private Debug _debug;

  /**
   * -lms von Kommandozeile
   */
  private String _lmsPid = null;
  
  /**
   * -loeschen von Kommandozeile
   */
  private int _loeschVerfahren = 0;
  
  /**
   * -ohneZeitstempelKorrektur von Kommandozeile
   */
  private int _ohneZeitstempelKorrektur = 0;
    
  public final static int OPTION_LOESCH_VERFAHREN_1 = 1;
  public final static int OPTION_LOESCH_VERFAHREN_2 = 2;
  public final static int OPTION_LOESCH_VERFAHREN_3 = 3;
  public final static int MASKE_LOESCH_VERFAHREN = 3;
  
  public final static int OPTION_OHNE_ZEITSTEMPEL_KORREKTUR = 4;
  public final static int MASKE_OHNE_ZEITSTEMPEL_KORREKTUR = 4;
  
  /**
   * Datenverteiler-Verbindung
   */
  private ClientDavInterface _connection;
   
  /**
   * Default- Konstruktor.<br>
   * Wird typischerweise von <code>main</code> aufgerufen
   */
  public RDSVerwaltungApp()
  {
    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
    }
  }
  
  /**
   * Extrahiert das Konfigurationsobjekt zur bergebenenen PID und fhrt ggf. 
   * eine Typprfung durch.
   * @param pid die PID des Konfigurationsobjektes, das extrahiert werden soll
   * @param typ der Typ, den das Konfigutationsobjekt haben muss - wenn hier <code>null</code> bergeben
   * wird, unterbleibt eine Typprfung.
   * @return das gefundene Konfigurationsobjekt
   * @throws IllegalArgumentException, falls Konfigurationsobjekt nicht gefunden oder 
   * vom falschen Typ
   */
  private ConfigurationObject getKonfigObjekt(String pid, String typ)
  {
    ConfigurationObject obj = null;
    
    try
    {
      obj = (ConfigurationObject)_connection.getDataModel().getObject(pid);
    }
    catch(ClassCastException e)
    {
      throw new IllegalArgumentException("Die PID \"" + pid + 
      "\" bezeichnet kein KonfigurationsObjekt");
    }
    
    if (obj == null)
    {
      throw new IllegalArgumentException("Zur PID \"" + pid + 
      "\" wurde kein Konfigurationsobjekt gefunden ");
    }
  
    if (typ != null && !obj.isOfType(typ))
    {
      throw new IllegalArgumentException("Das Konfigurationsobjekt \"" + 
          obj.getPid() + "\" ist nicht vom Typ " + typ); 
    }
    
    return obj;
  }
  
  /**
   * berschriebene Methode von <code>StandardApplication</code>, 
   * die die speziellen Startparameter auswertet.<br>
   * Die Argumtwerte, die ausgewertet werden, sind durch das Attribut 
   * {@link #_lmsPid} spezifiziert
   * 
   * @param argumentList
   *  siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList)
  {
    _debug = Debug.getLogger();
   
    _lmsPid = argumentList.fetchArgument("-lms=").asNonEmptyString();
    
    if (argumentList.hasArgument("-lschen"))
    {  
      _loeschVerfahren = argumentList.fetchArgument("-lschen=").intValueBetween(0, 1);
    }  
     
    // -loeschen statt -lschen, -lschen wird zur Rckwartskompatiblitt beibehalten,
    // aber -loeschen hat Vorrang
    if (argumentList.hasArgument("-loeschen"))
    {  
      _loeschVerfahren = argumentList.fetchArgument("-loeschen=").intValueBetween(0, 1);
    }  
    
    if (argumentList.hasArgument("-ohneZeitstempelKorrektur"))
    {  
      String text = argumentList.fetchArgument("-ohneZeitstempelKorrektur=").asString();
      if (text.equals("1") || text.equalsIgnoreCase("true") || text.equalsIgnoreCase("wahr"))
      {
        _ohneZeitstempelKorrektur = OPTION_OHNE_ZEITSTEMPEL_KORREKTUR;
      }
    } 
    
    argumentList.fetchUnusedArguments();
  }
  
    
  /**
   * berschriebene Methode von StandardApplication, die die Initialisierung durchfhrt.<br>
   * Es wird eine Instanz von <code>VerwaltungEngstellen</code> erzeugt und initialisiert
   * 
   * @param connection
   * siehe sys.funclib.application.StandardApplication#initialize(de.bsvrz.dav.daf.main.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection)
  {
    _connection = connection;
    try
    {
      MessageSender.getInstance().setApplicationLabel(
          _connection.getLocalApplicationObject().getName());
      
      ConfigurationObject lms = getKonfigObjekt(_lmsPid, "typ.landesMeldeStelle");
      
      final RDSVerwaltung verwaltung = new RDSVerwaltung(connection, lms, // Hauptapplikation
          _loeschVerfahren | _ohneZeitstempelKorrektur);
      
      Runtime.getRuntime().addShutdownHook(new Thread()  // Raeumt System auf
      {
        public void run()
        {
         _debug.info("Freigabe Resourcen ...");          
         verwaltung.dispose();  
        }
      });
      
      _debug.info("SWE RDS/TMC-Verwaltung initialisiert");
    }
    catch (Exception e)
    {
      _debug.error("Abbruch der SWE RDS/TMC-Verwaltung whrend der Initialisierung", e);
      System.exit(1);
    }
  }
  
  /**
   * Programmeinstieg RDS/TMC-Verwaltung.<br>
   * 
   * @param arguments Kommandozeilenargumente, folgende speziellen Argumente werden ausgewertet:<p>
   * -netz: Netz, dessen Infrastrukturobjekte verwendet werden.
   * -lms: Zustndige Landesmeldestelle (LMS). 
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new RDSVerwaltungApp(), arguments);
  }

}
